@@ -22,7 +22,7 @@ class GraphQLObject(BaseModel):
             },
         )
 
-    async def shrink(self):
+    async def ashrink(self):
         """WIll be called by the ward"""
         assert (
             self.id
@@ -48,6 +48,14 @@ class GraphQLInputObject(BaseModel):
             },
         )
 
+    async def ashrink(self):
+        """WIll be called by the ward"""
+        return self.dict()
+
+    def shrink(self):
+        """WIll be called by the ward"""
+        return self.dict()
+
     async def to_variable(self):
         """WIll be called by the ward"""
         dictionary = self.dict(exclude={"typename"})
